/*
 The analog comparator doesn't generate an interrupt, so you need to check for
 it after wakeup, for instance by reading the PWRDWN register. I have made a
 small example that show usage of the analog comparator.
 */
 
#include <Nordic\reg24le1.h>
#include "hal_clk.h"
#include "hal_rtc.h"
#include "hal_ancmp.h"

void mcu_init(void)
{
  P0CON = 0x70; // Disable digital input buffer, pin used for 32kHz.
  P0CON = 0x71; // Disable digital input buffer, pin used for 32kHz.
  P0CON = 0x73; // Disable digital input buffer, pin used for ADC

  P0DIR = 0x07;  // Two pins used for 32 kHz XO, one pin for ADC.
  P1DIR = 0x00;
  P2DIR = 0x00;
  P3DIR = 0x00;

  P0 = 0x00;
  P1 = 0x00;
  P2 = 0x00;
  P3 = 0x00;
}

// RTC wakeup interrupt every 1sec
void wakeup_tick() interrupt INTERRUPT_TICK
{
  P04 = !P04;
}

void main(void)
{
  uint16_t i;
  mcu_init();
  
  // The RTC is required for analog comparator operation.
  // In this case I also enable wakeup every 1sec.
  hal_rtc_start(false);
  hal_clklf_set_source(HAL_CLKLF_XOSC32K);          
  hal_rtc_set_compare_mode(HAL_RTC_COMPARE_MODE_0);
  hal_rtc_set_compare_value(0x7FFF);
  hal_rtc_start(true);              

  // Wait for the 32kHz to startup (change phase)
  while((CLKLFCTRL&0x80)==0x80);
  while((CLKLFCTRL&0x80)!=0x80);

  // Setup analog comparator
  hal_ancmp_set_polarity(HAL_ANCMP_NON_INVERTING);
  hal_ancmp_set_ref_voltage_scale(HAL_ANCMP_REF_SCALE_50);
  hal_ancmp_set_reference(HAL_ANCMP_REF_VDD);
  hal_ancmp_set_input_channel(HAL_ANCMP_INP_AIN2);
  hal_ancmp_enable(true);

  // Enable RTC wakeup
  IEN1 = 0x20;
  EA = 1;  
                  
  while(1)
  {
    // Enter register retention mode
    PWRDWN = 0x04;
    
    // If wakeup due to comparator
    if((PWRDWN & 0x20) == 0x20)
      P05 = 1;    
    // else if wakeup is due to RTC tick, then wait in standby mode for interrupt to occur (XO startup time)
    else 
      PWRDWN = 0x07;
    
    // Clear PWRDWN
    PWRDWN = 0x00;
    
    // Continue to run some dummy code 
    while(i--)
      ;

    // Disable analog comparator wakeup indication
    P05 = 0; 
  };
}


